@@ -1,15 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2018 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2024 -- leonerd@leonerd.org.uk
 
-package IO::Async::Stream;
+package IO::Async::Stream 0.804;
 
-use strict;
+use v5.14;
 use warnings;
 
-our $VERSION = '0.75';
-
 use base qw( IO::Async::Handle );
 
 use Errno qw( EAGAIN EWOULDBLOCK EINTR EPIPE );
@@ -19,7 +17,10 @@ use Carp;
 use Encode 2.11 qw( find_encoding STOP_AT_PARTIAL );
 use Scalar::Util qw( blessed );
 
+use Future 0.44; # ->result
+
 use IO::Async::Debug;
+use IO::Async::Metrics '$METRICS';
 
 # Tuneable from outside
 # Not yet documented
@@ -49,33 +50,33 @@ filehandle
 
 =head1 SYNOPSIS
 
- use IO::Async::Stream;
+   use IO::Async::Stream;
 
- use IO::Async::Loop;
- my $loop = IO::Async::Loop->new;
+   use IO::Async::Loop;
+   my $loop = IO::Async::Loop->new;
 
- my $stream = IO::Async::Stream->new(
-    read_handle  => \*STDIN,
-    write_handle => \*STDOUT,
+   my $stream = IO::Async::Stream->new(
+      read_handle  => \*STDIN,
+      write_handle => \*STDOUT,
 
-    on_read => sub {
-       my ( $self, $buffref, $eof ) = @_;
+      on_read => sub {
+         my ( $self, $buffref, $eof ) = @_;
 
-       while( $$buffref =~ s/^(.*\n)// ) {
-          print "Received a line $1";
-       }
+         while( $$buffref =~ s/^(.*\n)// ) {
+            print "Received a line $1";
+         }
 
-       if( $eof ) {
-          print "EOF; last partial line is $$buffref\n";
-       }
+         if( $eof ) {
+            print "EOF; last partial line is $$buffref\n";
+         }
 
-       return 0;
-    }
- );
+         return 0;
+      }
+   );
 
- $loop->add( $stream );
+   $loop->add( $stream );
 
- $stream->write( "An initial line here\n" );
+   $stream->write( "An initial line here\n" );
 
 =head1 DESCRIPTION
 
@@ -179,7 +180,7 @@ and re-enable notifications again once something has read enough to cause it to
 drop. If these events are overridden, the overriding code will have to perform
 this behaviour if required, by using
 
- $self->want_readready_for_read(...)
+   $self->want_readready_for_read(...);
 
 =head2 on_outgoing_empty
 
@@ -324,8 +325,8 @@ Optional. If defined, gives the name of a method or a CODE reference to use
 to implement the actual reading from or writing to the filehandle. These will
 be invoked as
 
- $stream->reader( $read_handle, $buffer, $len )
- $stream->writer( $write_handle, $buffer, $len )
+   $stream->reader( $read_handle, $buffer, $len );
+   $stream->writer( $write_handle, $buffer, $len );
 
 Each is expected to modify the passed buffer; C<reader> by appending to it,
 C<writer> by removing a prefix from it. Each is expected to return a true
@@ -391,11 +392,8 @@ sub configure
    }
 
    if( exists $params{read_high_watermark} or exists $params{read_low_watermark} ) {
-      my $high = delete $params{read_high_watermark};
-      defined $high or $high = $self->{read_high_watermark};
-
-      my $low  = delete $params{read_low_watermark};
-      defined $low  or $low  = $self->{read_low_watermark};
+      my $high = delete $params{read_high_watermark} // $self->{read_high_watermark};
+      my $low  = delete $params{read_low_watermark}  // $self->{read_low_watermark};
 
       croak "Cannot set read_low_watermark without read_high_watermark" if defined $low and !defined $high;
       croak "Cannot set read_high_watermark without read_low_watermark" if defined $high and !defined $low;
@@ -446,8 +444,8 @@ sub _add_to_loop
 
 =head1 METHODS
 
-The following methods documented with a trailing call to C<< ->get >> return
-L<Future> instances.
+The following methods documented in C<await> expressions return L<Future>
+instances.
 
 =cut
 
@@ -455,9 +453,9 @@ L<Future> instances.
 
 =head2 want_readready_for_write
 
-   $stream->want_readready_for_read( $set )
+   $stream->want_readready_for_read( $set );
 
-   $stream->want_readready_for_write( $set )
+   $stream->want_readready_for_write( $set );
 
 Mutators for the C<want_readready> property on L<IO::Async::Handle>, which
 control whether the C<read> or C<write> behaviour should be continued once the
@@ -494,9 +492,9 @@ sub want_readready_for_write
 
 =head2 want_writeready_for_write
 
-   $stream->want_writeready_for_write( $set )
+   $stream->want_writeready_for_write( $set );
 
-   $stream->want_writeready_for_read( $set )
+   $stream->want_writeready_for_read( $set );
 
 Mutators for the C<want_writeready> property on L<IO::Async::Handle>, which
 control whether the C<write> or C<read> behaviour should be continued once the
@@ -547,7 +545,7 @@ sub _is_empty
 
 =head2 close
 
-   $stream->close
+   $stream->close;
 
 A synonym for C<close_when_empty>. This should not be used when the deferred
 wait behaviour is required, as the behaviour of C<close> may change in a
@@ -563,15 +561,15 @@ sub close
 
 =head2 close_when_empty
 
-   $stream->close_when_empty
+   $stream->close_when_empty;
 
 If the write buffer is empty, this method calls C<close> on the underlying IO
 handles, and removes the stream from its containing loop. If the write buffer
 still contains data, then this is deferred until the buffer is empty. This is
 intended for "write-then-close" one-shot streams.
 
- $stream->write( "Here is my final data\n" );
- $stream->close_when_empty;
+   $stream->write( "Here is my final data\n" );
+   $stream->close_when_empty;
 
 Because of this deferred nature, it may not be suitable for error handling.
 See instead the C<close_now> method.
@@ -589,7 +587,7 @@ sub close_when_empty
 
 =head2 close_now
 
-   $stream->close_now
+   $stream->close_now;
 
 This method immediately closes the underlying IO handles and removes the
 stream from the containing loop. It will not wait to flush the remaining data
@@ -615,9 +613,9 @@ sub close_now
 
 =head2 is_write_eof
 
-   $eof = $stream->is_read_eof
+   $eof = $stream->is_read_eof;
 
-   $eof = $stream->is_write_eof
+   $eof = $stream->is_write_eof;
 
 Returns true after an EOF condition is reported on either the read or the
 write handle, respectively.
@@ -638,7 +636,7 @@ sub is_write_eof
 
 =head2 write
 
-   $stream->write( $data, %params )
+   $stream->write( $data, %params );
 
 This method adds data to the outgoing data queue, or writes it immediately,
 according to the C<autoflush> parameter.
@@ -667,14 +665,14 @@ C<Future>s, as well as plain strings.
 
 For example, to stream the contents of an existing opened filehandle:
 
- open my $fileh, "<", $path or die "Cannot open $path - $!";
+   open my $fileh, "<", $path or die "Cannot open $path - $!";
 
- $stream->write( sub {
-    my ( $stream ) = @_;
+   $stream->write( sub {
+      my ( $stream ) = @_;
 
-    sysread $fileh, my $buffer, 8192 or return;
-    return $buffer;
- } );
+      sysread $fileh, my $buffer, 8192 or return;
+      return $buffer;
+   } );
 
 Takes the following optional named parameters in C<%params>:
 
@@ -692,7 +690,7 @@ that were written by this call, which may not be the entire length of the
 buffer - if it takes more than one C<syscall> operation to empty the buffer
 then this callback will be invoked multiple times.
 
- $on_write->( $stream, $len )
+   $on_write->( $stream, $len );
 
 =item on_flush => CODE
 
@@ -700,14 +698,14 @@ A CODE reference which will be invoked once the data queued by this C<write>
 call has been flushed. This will be invoked even if the buffer itself is not
 yet empty; if more data has been queued since the call.
 
- $on_flush->( $stream )
+   $on_flush->( $stream );
 
 =item on_error => CODE
 
 A CODE reference which will be invoked if a C<syswrite> error happens while
 performing this write. Invoked as for the C<Stream>'s C<on_write_error> event.
 
- $on_error->( $stream, $errno )
+   $on_error->( $stream, $errno );
 
 =back
 
@@ -722,7 +720,7 @@ to this point.
 
 =head2 write (scalar)
 
-   $stream->write( ... )->get
+   await $stream->write( ... );
 
 If called in non-void context, this method returns a L<Future> which will
 complete (with no value) when the write operation has been flushed. This may
@@ -773,7 +771,7 @@ sub _flush_one_write
             $head->watching++;
             return 0;
          }
-         my $data = $f->get;
+         my $data = $f->result;
          if( !ref $data and my $encoding = $self->{encoding} ) {
             $data = $encoding->encode( $data );
          }
@@ -833,6 +831,8 @@ sub _flush_one_write
       return 0;
    }
 
+   $METRICS and $METRICS->inc_counter_by( stream_written => $len ) if $len;
+
    if( my $on_write = $head->on_write ) {
       $on_write->( $self, $len );
    }
@@ -888,8 +888,7 @@ sub write
       };
    }
 
-   my $write_len = $params{write_len};
-   defined $write_len or $write_len = $self->{write_len};
+   my $write_len = $params{write_len} // $self->{write_len};
 
    push @{ $self->{writequeue} }, Writer(
       $data, $write_len, $on_write, $on_flush, $on_error, 0
@@ -1027,6 +1026,8 @@ sub _do_read
          IO::Async::Debug::log_hexdump( $data ) if $IO::Async::Debug::DEBUG_FLAGS{Sr};
       }
 
+      $METRICS and $METRICS->inc_counter_by( stream_read => $len ) if $len;
+
       my $eof = $self->{read_eof} = ( $len == 0 );
 
       if( my $encoding = $self->{encoding} ) {
@@ -1075,7 +1076,7 @@ sub on_read_low_watermark
 
 =head2 push_on_read
 
-   $stream->push_on_read( $on_read )
+   $stream->push_on_read( $on_read );
 
 Pushes a new temporary C<on_read> handler to the end of the queue. This queue,
 if non-empty, is used to provide C<on_read> event handling code in preference
@@ -1112,9 +1113,7 @@ enough data has been read by the Stream into its buffer. At this point, the
 data is removed from the buffer and given to the C<Future> object to complete
 it.
 
- my $f = $stream->read_...
-
- my ( $string ) = $f->get;
+   my $string = await $stream->read_...
 
 Unlike the C<on_read> event handlers, these methods don't allow for access to
 "partial" results; they only provide the final result once it is ready.
@@ -1128,17 +1127,17 @@ C<Future>-returning methods instead of the C<on_read> event, it may be useful
 to configure a trivial return-false event handler to keep it from consuming
 any input, and to allow it to be added to a C<Loop> in the first place.
 
- my $stream = IO::Async::Stream->new( on_read => sub { 0 }, ... );
- $loop->add( $stream );
+   my $stream = IO::Async::Stream->new( on_read => sub { 0 }, ... );
+   $loop->add( $stream );
 
- my $f = $stream->read_...
+   my $f = $stream->read_...
 
 If a read EOF or error condition happens while there are read C<Future>s
 pending, they are all completed. In the case of a read EOF, they are done with
 C<undef>; in the case of a read error they are failed using the C<$!> error
 value as the failure.
 
- $f->fail( $message, sysread => $! )
+   $f->fail( $message, sysread => $! )
 
 If a read EOF condition happens to the currently-processing read C<Future>, it
 will return a partial result. The calling code can detect this by the fact
@@ -1147,7 +1146,7 @@ short in C<read_exactly>'s case, or lacking the ending pattern in
 C<read_until>'s case). Additionally, each C<Future> will yield the C<$eof>
 value in its results.
 
- my ( $string, $eof ) = $f->get;
+   my ( $string, $eof ) = await ...;
 
 =cut
 
@@ -1166,9 +1165,9 @@ sub _read_future
 
 =head2 read_exactly
 
-   ( $string, $eof ) = $stream->read_atmost( $len )->get
+   ( $string, $eof ) = await $stream->read_atmost( $len );
 
-   ( $string, $eof ) = $stream->read_exactly( $len )->get
+   ( $string, $eof ) = await $stream->read_exactly( $len );
 
 Completes the C<Future> when the read buffer contains C<$len> or more
 characters of input. C<read_atmost> will also complete after the first
@@ -1210,7 +1209,7 @@ sub read_exactly
 
 =head2 read_until
 
-   ( $string, $eof ) = $stream->read_until( $end )->get
+   ( $string, $eof ) = await $stream->read_until( $end );
 
 Completes the C<Future> when the read buffer contains a match for C<$end>,
 which may either be a plain string or a compiled C<Regexp> reference. Yields
@@ -1246,7 +1245,7 @@ sub read_until
 
 =head2 read_until_eof
 
-   ( $string, $eof ) = $stream->read_until_eof->get
+   ( $string, $eof ) = await $stream->read_until_eof;
 
 Completes the C<Future> when the stream is eventually closed at EOF, and
 yields all of the data that was available.
@@ -1278,11 +1277,11 @@ sub read_until_eof
 
 =head2 new_for_stdio
 
-   $stream = IO::Async::Stream->new_for_stdin
+   $stream = IO::Async::Stream->new_for_stdin;
 
-   $stream = IO::Async::Stream->new_for_stdout
+   $stream = IO::Async::Stream->new_for_stdout;
 
-   $stream = IO::Async::Stream->new_for_stdio
+   $stream = IO::Async::Stream->new_for_stdio;
 
 Return a C<IO::Async::Stream> object preconfigured with the correct
 C<read_handle>, C<write_handle> or both.
@@ -1296,7 +1295,7 @@ sub new_for_stdio { shift->new( read_handle => \*STDIN, write_handle => \*STDOUT
 
 =head2 connect
 
-   $future = $stream->connect( %args )
+   $future = $stream->connect( %args );
 
 A convenient wrapper for calling the C<connect> method on the underlying
 L<IO::Async::Loop> object, passing the C<socktype> hint as C<stream> if not
@@ -1335,17 +1334,17 @@ Log byte buffers as data is written to a Stream
 The following C<on_read> method accepts incoming C<\n>-terminated lines and
 prints them to the program's C<STDOUT> stream.
 
- sub on_read
- {
-    my $self = shift;
-    my ( $buffref, $eof ) = @_;
+   sub on_read
+   {
+      my $self = shift;
+      my ( $buffref, $eof ) = @_;
 
-    while( $$buffref =~ s/^(.*\n)// ) {
-       print "Received a line: $1";
-    }
+      while( $$buffref =~ s/^(.*\n)// ) {
+         print "Received a line: $1";
+      }
 
-    return 0;
- }
+      return 0;
+   }
 
 Because a reference to the buffer itself is passed, it is simple to use a
 C<s///> regular expression on the scalar it points at, to both check if data
@@ -1358,23 +1357,23 @@ again when it has finished, so it can return a constant C<0>.
 This C<on_read> method accepts incoming records in 16-byte chunks, printing
 each one.
 
- sub on_read
- {
-    my ( $self, $buffref, $eof ) = @_;
+   sub on_read
+   {
+      my ( $self, $buffref, $eof ) = @_;
 
-    if( length $$buffref >= 16 ) {
-       my $record = substr( $$buffref, 0, 16, "" );
-       print "Received a 16-byte record: $record\n";
+      if( length $$buffref >= 16 ) {
+         my $record = substr( $$buffref, 0, 16, "" );
+         print "Received a 16-byte record: $record\n";
 
-       return 1;
-    }
+         return 1;
+      }
 
-    if( $eof and length $$buffref ) {
-       print "EOF: a partial record still exists\n";
-    }
+      if( $eof and length $$buffref ) {
+         print "EOF: a partial record still exists\n";
+      }
 
-    return 0;
- }
+      return 0;
+   }
 
 This time, rather than a C<while()> loop we have decided to have the handler
 just process one record, and use the C<return 1> mechanism to ask that the
@@ -1389,23 +1388,23 @@ A lot of protocols use a fixed-size header, followed by a variable-sized body
 of data, whose size is given by one of the fields of the header. The following
 C<on_read> method extracts messages in such a protocol.
 
- sub on_read
- {
-    my ( $self, $buffref, $eof ) = @_;
+   sub on_read
+   {
+      my ( $self, $buffref, $eof ) = @_;
 
-    return 0 unless length $$buffref >= 8; # "N n n" consumes 8 bytes
+      return 0 unless length $$buffref >= 8; # "N n n" consumes 8 bytes
 
-    my ( $len, $x, $y ) = unpack "N n n", $$buffref;
+      my ( $len, $x, $y ) = unpack "N n n", $$buffref;
 
-    return 0 unless length $$buffref >= 8 + $len;
+      return 0 unless length $$buffref >= 8 + $len;
 
-    substr( $$buffref, 0, 8, "" );
-    my $data = substr( $$buffref, 0, $len, "" );
+      substr( $$buffref, 0, 8, "" );
+      my $data = substr( $$buffref, 0, $len, "" );
 
-    print "A record with values x=$x y=$y\n";
+      print "A record with values x=$x y=$y\n";
 
-    return 1;
- }
+      return 1;
+   }
 
 In this example, the header is C<unpack()>ed first, to extract the body
 length, and then the body is extracted. If the buffer does not have enough
@@ -1420,41 +1419,41 @@ C<\n>-terminated lines that may have an optional data block attached. The
 presence of such a data block, as well as its size, is indicated by the line
 prefix.
 
- sub on_read
- {
-    my $self = shift;
-    my ( $buffref, $eof ) = @_;
+   sub on_read
+   {
+      my $self = shift;
+      my ( $buffref, $eof ) = @_;
 
-    if( $$buffref =~ s/^DATA (\d+):(.*)\n// ) {
-       my $length = $1;
-       my $line   = $2;
+      if( $$buffref =~ s/^DATA (\d+):(.*)\n// ) {
+         my $length = $1;
+         my $line   = $2;
 
-       return sub {
-          my $self = shift;
-          my ( $buffref, $eof ) = @_;
+         return sub {
+            my $self = shift;
+            my ( $buffref, $eof ) = @_;
 
-          return 0 unless length $$buffref >= $length;
+            return 0 unless length $$buffref >= $length;
 
-          # Take and remove the data from the buffer
-          my $data = substr( $$buffref, 0, $length, "" );
+            # Take and remove the data from the buffer
+            my $data = substr( $$buffref, 0, $length, "" );
 
-          print "Received a line $line with some data ($data)\n";
+            print "Received a line $line with some data ($data)\n";
 
-          return undef; # Restore the original method
-       }
-    }
-    elsif( $$buffref =~ s/^LINE:(.*)\n// ) {
-       my $line = $1;
+            return undef; # Restore the original method
+         }
+      }
+      elsif( $$buffref =~ s/^LINE:(.*)\n// ) {
+         my $line = $1;
 
-       print "Received a line $line with no data\n";
+         print "Received a line $line with no data\n";
 
-       return 1;
-    }
-    else {
-       print STDERR "Unrecognised input\n";
-       # Handle it somehow
-    }
- }
+         return 1;
+      }
+      else {
+         print STDERR "Unrecognised input\n";
+         # Handle it somehow
+      }
+   }
 
 In the case where trailing data is supplied, a new temporary C<on_read>
 callback is provided in a closure. This closure captures the C<$length>
