@@ -1,16 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2024 -- leonerd@leonerd.org.uk
 
-package IO::Async::Timer::Absolute;
+package IO::Async::Timer::Absolute 0.804;
 
-use strict;
+use v5.14;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.78';
-
 use Carp;
 
 =head1 NAME
@@ -19,27 +17,27 @@ C<IO::Async::Timer::Absolute> - event callback at a fixed future time
 
 =head1 SYNOPSIS
 
- use IO::Async::Timer::Absolute;
+   use IO::Async::Timer::Absolute;
 
- use POSIX qw( mktime );
+   use POSIX qw( mktime );
 
- use IO::Async::Loop;
- my $loop = IO::Async::Loop->new;
+   use IO::Async::Loop;
+   my $loop = IO::Async::Loop->new;
 
- my @time = gmtime;
+   my @time = gmtime;
 
- my $timer = IO::Async::Timer::Absolute->new(
-    time => mktime( 0, 0, 0, $time[3]+1, $time[4], $time[5] ),
+   my $timer = IO::Async::Timer::Absolute->new(
+      time => mktime( 0, 0, 0, $time[3]+1, $time[4], $time[5] ),
 
-    on_expire => sub {
-       print "It's midnight\n";
-       $loop->stop;
-    },
- );
+      on_expire => sub {
+         print "It's midnight\n";
+         $loop->stop;
+      },
+   );
 
- $loop->add( $timer );
+   $loop->add( $timer );
 
- $loop->run;
+   $loop->run;
 
 =head1 DESCRIPTION
 
